package org.aleabodo.fireworksmc;

import org.bukkit.Bukkit;
import org.bukkit.Color;
import org.bukkit.FireworkEffect;
import org.bukkit.Location;
import org.bukkit.entity.EntityType;
import org.bukkit.entity.Firework;
import org.bukkit.inventory.meta.FireworkMeta;
import org.bukkit.plugin.Plugin;
import org.bukkit.scheduler.BukkitRunnable;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Random;

public class Show {
    private String seed;
    private Location location;
    private Direction direction;
    private int duration; //in seconds

    public Show(String seed, Location location, Direction direction, int duration) {
        this.seed = seed;
        this.location = location;
        this.direction = direction;
        this.duration = duration;
    }

    /**
     * Executes the firework show
     */
    public void start() {
        spawnRocket(this.location);
    }

    public void spawnRocket(Location location) {
        Firework firework = (Firework) location.getWorld().spawnEntity(location, EntityType.FIREWORK);
        FireworkMeta fireworkMeta = firework.getFireworkMeta();

        // Create a firework effect
        FireworkEffect effect = FireworkEffect.builder()
                .flicker(true)
                .trail(true)
                .withColor(Color.RED)
                .withFade(Color.YELLOW)
                .with(FireworkEffect.Type.BALL_LARGE)
                .build();

        fireworkMeta.addEffect(effect);
        fireworkMeta.setPower(1); // Number of times the firework will explode
        firework.setFireworkMeta(fireworkMeta);

        // Schedule the firework to be removed after a delay
        new BukkitRunnable() {
            @Override
            public void run() {
                firework.detonate();
            }
        }.runTaskLater(Fireworks_Mc.getPlugin(), 20); // 20 ticks = 1 second
    }

    /**
     *
     * @param seedString Random seed as string
     * @return Random number between 0 and based on the seed
     */
    private double generateRandomValueFromSeed(String seedString) {
        try {
            MessageDigest md = MessageDigest.getInstance("SHA-256");
            byte[] hashBytes = md.digest(seedString.getBytes());

            // Convert the first 8 bytes of the hash to a long value
            long hashValue = 0;
            for (int i = 0; i < 8; i++) {
                hashValue = (hashValue << 8) | (hashBytes[i]);
            }

            //Convert long (hastValue) into random number between 0 and 1.
            return new Random(hashValue).nextDouble();
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException("Hashing algorithm not available. (Have to check or Intellij complains.", e);
        }
    }

    /*
     * ################### GETTER ###################
     */

    public String getSeed() {
        return seed;
    }

    public Location getLocation() {
        return location;
    }

    public Direction getDirection() {
        return direction;
    }

    public int getDuration() {
        return duration;
    }



    /*
     * ################### SETTER ###################
     */

    public void setSeed(String seed) {
        this.seed = seed;
    }

    public void setLocation(Location location) {
        this.location = location;
    }

    public void setDirection(Direction direction) {
        this.direction = direction;
    }

    public void setDuration(int duration) {
        this.duration = duration;
    }
}
